@@ -22,7 +22,7 @@ use strict;
 use vars qw($AUTOLOAD $VERSION $ABSTRACT @ISA @EXPORT);
 
 BEGIN {
-	$VERSION = 0.35;
+	$VERSION = 1.02;
 	$ABSTRACT = "Fast HTML Parser using Threads with no outside dependencies";
 	
 	@ISA = qw(Exporter DynaLoader);
@@ -35,7 +35,8 @@ BEGIN {
 		MyHTML_ENCODING_ISO_8859_8 MyHTML_ENCODING_KOI8_R MyHTML_ENCODING_KOI8_U MyHTML_ENCODING_MACINTOSH
 		MyHTML_ENCODING_WINDOWS_1250 MyHTML_ENCODING_WINDOWS_1251 MyHTML_ENCODING_WINDOWS_1252 MyHTML_ENCODING_WINDOWS_1253
 		MyHTML_ENCODING_WINDOWS_1254 MyHTML_ENCODING_WINDOWS_1255 MyHTML_ENCODING_WINDOWS_1256 MyHTML_ENCODING_WINDOWS_1257
-		MyHTML_ENCODING_WINDOWS_1258 MyHTML_ENCODING_WINDOWS_874 MyHTML_ENCODING_X_MAC_CYRILLIC MyHTML_ENCODING_LAST_ENTRY
+		MyHTML_ENCODING_WINDOWS_1258 MyHTML_ENCODING_WINDOWS_874 MyHTML_ENCODING_X_MAC_CYRILLIC MyHTML_ENCODING_ISO_2022_JP
+		MyHTML_ENCODING_GBK MyHTML_ENCODING_SHIFT_JIS MyHTML_ENCODING_EUC_JP MyHTML_ENCODING_ISO_8859_8_I MyHTML_ENCODING_LAST_ENTRY 
 		
 		MyHTML_TAG__UNDEF MyHTML_TAG__TEXT MyHTML_TAG__COMMENT MyHTML_TAG__DOCTYPE MyHTML_TAG_A MyHTML_TAG_ABBR
 		MyHTML_TAG_ACRONYM MyHTML_TAG_ADDRESS MyHTML_TAG_ANNOTATION_XML MyHTML_TAG_APPLET MyHTML_TAG_AREA MyHTML_TAG_ARTICLE
@@ -78,23 +79,32 @@ BEGIN {
 		MyHTML_TAG_MSLINE MyHTML_TAG_MSPACE MyHTML_TAG_MSQRT MyHTML_TAG_MSROW MyHTML_TAG_MSTACK MyHTML_TAG_MSTYLE
 		MyHTML_TAG_MSUB MyHTML_TAG_MSUP MyHTML_TAG_MSUBSUP MyHTML_TAG__END_OF_FILE MyHTML_TAG_FIRST_ENTRY MyHTML_TAG_LAST_ENTRY
 		
+		MyHTML_TREE_PARSE_FLAGS_CLEAN MyHTML_TREE_PARSE_FLAGS_WITHOUT_BUILD_TREE MyHTML_TREE_PARSE_FLAGS_WITHOUT_PROCESS_TOKEN
+		MyHTML_TREE_PARSE_FLAGS_SKIP_WHITESPACE_TOKEN MyHTML_TREE_PARSE_FLAGS_WITHOUT_DOCTYPE_IN_TREE 
+		
 		MyHTML_NAMESPACE_UNDEF MyHTML_NAMESPACE_HTML MyHTML_NAMESPACE_MATHML MyHTML_NAMESPACE_SVG MyHTML_NAMESPACE_XLINK MyHTML_NAMESPACE_XML
 		MyHTML_NAMESPACE_XMLNS MyHTML_NAMESPACE_LAST_ENTRY 
 		
-		MyHTML_STATUS_OK MyHTML_STATUS_ERROR_MEMORY_ALLOCATION MyHTML_STATUS_THREAD_ERROR_LIST_INIT MyHTML_STATUS_THREAD_ERROR_ATTR_MALLOC
-		MyHTML_STATUS_THREAD_ERROR_ATTR_INIT MyHTML_STATUS_THREAD_ERROR_ATTR_SET MyHTML_STATUS_THREAD_ERROR_ATTR_DESTROY
-		MyHTML_STATUS_THREAD_ERROR_NO_SLOTS MyHTML_STATUS_THREAD_ERROR_BATCH_INIT MyHTML_STATUS_THREAD_ERROR_WORKER_MALLOC
-		MyHTML_STATUS_THREAD_ERROR_WORKER_SEM_CREATE MyHTML_STATUS_THREAD_ERROR_WORKER_THREAD_CREATE MyHTML_STATUS_THREAD_ERROR_MASTER_THREAD_CREATE
-		MyHTML_STATUS_THREAD_ERROR_SEM_PREFIX_MALLOC MyHTML_STATUS_THREAD_ERROR_SEM_CREATE MyHTML_STATUS_THREAD_ERROR_QUEUE_MALLOC
-		MyHTML_STATUS_THREAD_ERROR_QUEUE_NODES_MALLOC MyHTML_STATUS_THREAD_ERROR_QUEUE_NODE_MALLOC MyHTML_STATUS_RULES_ERROR_MEMORY_ALLOCATION
+		MyHTML_STATUS_OK MyHTML_STATUS_ERROR_MEMORY_ALLOCATION MyHTML_STATUS_THREAD_ERROR_MEMORY_ALLOCATION MyHTML_STATUS_THREAD_ERROR_LIST_INIT
+		MyHTML_STATUS_THREAD_ERROR_ATTR_MALLOC MyHTML_STATUS_THREAD_ERROR_ATTR_INIT MyHTML_STATUS_THREAD_ERROR_ATTR_SET MyHTML_STATUS_THREAD_ERROR_ATTR_DESTROY
+		MyHTML_STATUS_THREAD_ERROR_NO_SLOTS MyHTML_STATUS_THREAD_ERROR_BATCH_INIT MyHTML_STATUS_THREAD_ERROR_WORKER_MALLOC MyHTML_STATUS_THREAD_ERROR_WORKER_SEM_CREATE
+		MyHTML_STATUS_THREAD_ERROR_WORKER_THREAD_CREATE MyHTML_STATUS_THREAD_ERROR_MASTER_THREAD_CREATE MyHTML_STATUS_THREAD_ERROR_SEM_PREFIX_MALLOC
+		MyHTML_STATUS_THREAD_ERROR_SEM_CREATE MyHTML_STATUS_THREAD_ERROR_QUEUE_MALLOC MyHTML_STATUS_THREAD_ERROR_QUEUE_NODES_MALLOC
+		MyHTML_STATUS_THREAD_ERROR_QUEUE_NODE_MALLOC MyHTML_STATUS_THREAD_ERROR_MUTEX_MALLOC MyHTML_STATUS_THREAD_ERROR_MUTEX_INIT
+		MyHTML_STATUS_THREAD_ERROR_MUTEX_LOCK MyHTML_STATUS_THREAD_ERROR_MUTEX_UNLOCK MyHTML_STATUS_RULES_ERROR_MEMORY_ALLOCATION
 		MyHTML_STATUS_PERF_ERROR_COMPILED_WITHOUT_PERF MyHTML_STATUS_PERF_ERROR_FIND_CPU_CLOCK MyHTML_STATUS_TOKENIZER_ERROR_MEMORY_ALLOCATION
-		MyHTML_STATUS_TAGS_ERROR_MEMORY_ALLOCATION MyHTML_STATUS_TAGS_ERROR_MCOBJECT_CREATE MyHTML_STATUS_TAGS_ERROR_MCOBJECT_MALLOC
-		MyHTML_STATUS_TAGS_ERROR_MCOBJECT_CREATE_NODE MyHTML_STATUS_TAGS_ERROR_CACHE_MEMORY_ALLOCATION MyHTML_STATUS_TAGS_ERROR_INDEX_MEMORY_ALLOCATION
-		MyHTML_STATUS_TREE_ERROR_MEMORY_ALLOCATION MyHTML_STATUS_TREE_ERROR_MCOBJECT_CREATE MyHTML_STATUS_TREE_ERROR_MCOBJECT_INIT
-		MyHTML_STATUS_TREE_ERROR_MCOBJECT_CREATE_NODE MyHTML_STATUS_ATTR_ERROR_ALLOCATION MyHTML_STATUS_ATTR_ERROR_CREATE
+		MyHTML_STATUS_TOKENIZER_ERROR_FRAGMENT_INIT MyHTML_STATUS_TAGS_ERROR_MEMORY_ALLOCATION MyHTML_STATUS_TAGS_ERROR_MCOBJECT_CREATE
+		MyHTML_STATUS_TAGS_ERROR_MCOBJECT_MALLOC MyHTML_STATUS_TAGS_ERROR_MCOBJECT_CREATE_NODE MyHTML_STATUS_TAGS_ERROR_CACHE_MEMORY_ALLOCATION
+		MyHTML_STATUS_TAGS_ERROR_INDEX_MEMORY_ALLOCATION MyHTML_STATUS_TREE_ERROR_MEMORY_ALLOCATION MyHTML_STATUS_TREE_ERROR_MCOBJECT_CREATE
+		MyHTML_STATUS_TREE_ERROR_MCOBJECT_INIT MyHTML_STATUS_TREE_ERROR_MCOBJECT_CREATE_NODE MyHTML_STATUS_TREE_ERROR_INCOMING_BUFFER_CREATE
+		MyHTML_STATUS_ATTR_ERROR_ALLOCATION MyHTML_STATUS_ATTR_ERROR_CREATE MyHTML_STATUS_STREAM_BUFFER_ERROR_CREATE MyHTML_STATUS_STREAM_BUFFER_ERROR_INIT
+		MyHTML_STATUS_STREAM_BUFFER_ENTRY_ERROR_CREATE MyHTML_STATUS_STREAM_BUFFER_ENTRY_ERROR_INIT MyHTML_STATUS_STREAM_BUFFER_ERROR_ADD_ENTRY
+		MyHTML_STATUS_MCOBJECT_ERROR_CACHE_CREATE MyHTML_STATUS_MCOBJECT_ERROR_CHUNK_CREATE MyHTML_STATUS_MCOBJECT_ERROR_CHUNK_INIT
+		MyHTML_STATUS_MCOBJECT_ERROR_CACHE_REALLOC
 		
 		MyHTML_OPTIONS_DEFAULT MyHTML_OPTIONS_PARSE_MODE_SINGLE MyHTML_OPTIONS_PARSE_MODE_ALL_IN_ONE MyHTML_OPTIONS_PARSE_MODE_SEPARATELY
-		MyHTML_OPTIONS_PARSE_MODE_WORKER_TREE MyHTML_OPTIONS_PARSE_MODE_WORKER_INDEX MyHTML_OPTIONS_PARSE_MODE_TREE_INDEX 
+		
+		namespace_name_by_id namespace_id_by_name
 	);
 };
 
@@ -114,7 +124,7 @@ HTML::MyHTML is a fast HTML Parser using Threads with no outside dependencies
 
 =head1 DESCRIPTION
 
-This Parser based on L<MyHTML library|https://github.com/lexborisov/myhtml>  (it includes version 0.5.3)
+This Parser based on L<MyHTML library|https://github.com/lexborisov/myhtml>  (it includes version 1.0.2)
 
 =over 4
 
@@ -166,6 +176,7 @@ See latest version on L<https://github.com/lexborisov/perl-html-myhtml|https://g
  
  print "\nGet all DIV elements of HTML Tree:\n";
  my $list = $tree->get_elements_by_tag_name("div");
+ # or my $list = $tree->body()->get_nodes_by_tag_id($tree, MyHTML_TAG_DIV);
  
  foreach my $node (@$list) {
  	my $info = $node->info($tree);
@@ -347,6 +358,25 @@ Destroy of a MyHTML_TREE structure
 Return: UNDEF if successful, otherwise an HTML::MyHTML::Tree structure
 
 
+=head3 parse_flags_set
+
+Set Parse Flags for Tree
+
+ $tree->parse_flags_set($parse_flags);
+
+Example:
+
+ $tree->parse_flags_set( MyHTML_TREE_PARSE_FLAGS_WITHOUT_BUILD_TREE|MyHTML_TREE_PARSE_FLAGS_WITHOUT_DOCTYPE_IN_TREE|MyHTML_TREE_PARSE_FLAGS_SKIP_WHITESPACE_TOKEN );
+
+
+=head3 parse_flags
+
+Get Parse Flags of Tree
+
+ my $parse_flags = $tree->parse_flags();
+
+Return: myhtml_tree_parse_flags_t
+
 =head3 get_myhtml
 
 Get HTML::MyHTML from a HTML::MyHTML::Tree object
@@ -438,6 +468,51 @@ Return: array list of elements HTML::MyHTML::Tree::Node
 Return: array list of elements HTML::MyHTML::Tree::Node
 
 
+=head3 callback_before_token_done_set
+
+Set callback for tokens before processing.
+
+Important!!! Only for Perl! Do not use this callback in Thread mode parsing; Build without threads or use methods parse_single, parse_fragment_single, parse_chunk_single, parse_chunk_fragment_single or create myhtml with MyHTML_OPTIONS_PARSE_MODE_SINGLE option; 
+
+ $tree->callback_before_token_done_set($sub_callback [, $ctx]);
+
+
+=head3 callback_after_token_done_set
+
+Set callback for tokens after processing
+
+Important!!! Only for Perl! Do not use this callback in Thread mode parsing; Build without threads or use methods parse_single, parse_fragment_single, parse_chunk_single, parse_chunk_fragment_single or create myhtml with MyHTML_OPTIONS_PARSE_MODE_SINGLE option; 
+
+ $tree->callback_after_token_done_set($sub_callback [, $ctx]);
+
+
+=head3 callback_node_insert_set
+
+Set callback for tree node after inserted
+
+Important!!! Only for Perl! Do not use this callback in Thread mode parsing; Build without threads or use methods parse_single, parse_fragment_single, parse_chunk_single, parse_chunk_fragment_single or create myhtml with MyHTML_OPTIONS_PARSE_MODE_SINGLE option; 
+
+ $tree->callback_node_insert_set($sub_callback [, $ctx]);
+
+
+=head3 callback_node_remove_set
+
+Set callback for tree node after removed
+
+Important!!! Only for Perl! Do not use this callback in Thread mode parsing; Build without threads or use methods parse_single, parse_fragment_single, parse_chunk_single, parse_chunk_fragment_single or create myhtml with MyHTML_OPTIONS_PARSE_MODE_SINGLE option; 
+
+ $tree->callback_node_remove_set($sub_callback [, $ctx]);
+
+
+=head3 incoming_buffer_first
+
+Get first Incoming Buffer
+
+ my $incoming_buffer = $tree->incoming_buffer_first();
+
+Return: HTML::Incoming::Buffer if exists, otherwise an UNDEF value
+
+
 =head2 Attributes
 
 =head3 info
@@ -571,17 +646,110 @@ Get last child of node
 Return: HTML::MyHTML::Tree::Node if exists, otherwise an UNDEF value
 
 
+=head3 token
+
+Get token node
+
+ my $token_node = $node->token();
+
+Return: HTML::MyHTML::Token::Node if exists, otherwise an UNDEF value
+
+
+=head3 get_nodes_by_attribute_key
+
+Get nodes by attribute key of current node
+
+ my $nodes = $node->get_nodes_by_attribute_key($tree, $key [, $out_status]);
+
+Return: HTML::MyHTML::Tree::Node ARRAY if exists, otherwise an UNDEF value
+
+
+=head3 get_nodes_by_attribute_value
+
+Get nodes by attribute value; exactly equal; like a [foo="bar"]
+
+ # $case_insensitive: 1 or 0
+ # $key: may bу undef for find in all keys
+ my $nodes = $node->get_nodes_by_attribute_value($tree, $case_insensitive, $key, $value [, $out_status]);
+
+Return: HTML::MyHTML::Tree::Node ARRAY if exists, otherwise an UNDEF value
+
+
+=head3 get_nodes_by_attribute_value_whitespace_separated
+
+Get nodes by attribute value; whitespace separated; like a [foo~="bar"]
+
+ # $case_insensitive: 1 or 0
+ # $key: may bу undef for find in all keys
+ my $nodes = $node->get_nodes_by_attribute_value_whitespace_separated($tree, $case_insensitive, $key, $value [, $out_status]);
+
+Return: HTML::MyHTML::Tree::Node ARRAY if exists, otherwise an UNDEF value
+
+
+=head3 get_nodes_by_attribute_value_begin
+
+Get nodes by attribute value; value begins exactly with the string; like a [foo^="bar"]
+
+ # $case_insensitive: 1 or 0
+ # $key: may bу undef for find in all keys
+ my $nodes = $node->get_nodes_by_attribute_value_begin($tree, $case_insensitive, $key, $value [, $out_status]);
+
+Return: HTML::MyHTML::Tree::Node ARRAY if exists, otherwise an UNDEF value
+
+
+=head3 get_nodes_by_attribute_value_end
+
+Get nodes by attribute value; value ends exactly with the string; like a [foo$="bar"]
+
+ # $case_insensitive: 1 or 0
+ # $key: may bу undef for find in all keys
+ my $nodes = $node->get_nodes_by_attribute_value_end($tree, $case_insensitive, $key, $value [, $out_status]);
+
+Return: HTML::MyHTML::Tree::Node ARRAY if exists, otherwise an UNDEF value
+
+
+=head3 get_nodes_by_attribute_value_contain
+
+Get nodes by attribute value; value contains the substring; like a [foo*="bar"]
+
+ # $case_insensitive: 1 or 0
+ # $key: may bу undef for find in all keys
+ my $nodes = $node->get_nodes_by_attribute_value_contain($tree, $case_insensitive, $key, $value [, $out_status]);
+
+Return: HTML::MyHTML::Tree::Node ARRAY if exists, otherwise an UNDEF value
+
+
+=head3 get_nodes_by_attribute_value_hyphen_separated
+
+Get nodes by attribute value; attribute value is a hyphen-separated list of values beginning
+
+ # $case_insensitive: 1 or 0
+ # $key: may bу undef for find in all keys
+ my $nodes = $node->get_nodes_by_attribute_value_hyphen_separated($tree, $case_insensitive, $key, $value [, $out_status]);
+
+Return: HTML::MyHTML::Tree::Node ARRAY if exists, otherwise an UNDEF value
+
+
+=head3 get_nodes_by_tag_id
+
+Get nodes by tag id in node scope
+
+ my $nodes = $node->get_nodes_by_tag_id($tree, $tag_id [, $out_status]);
+
+Return: HTML::MyHTML::Tree::Node ARRAY if exists, otherwise an UNDEF value
+
+
 =head3 free
 
 Release allocated resources
 
- $node->free($node, $tree);
+ $node->free($tree);
 
 =head3 remove
 
 Remove node of tree
 
- my $node = $node->remove();
+ my $node = $node->remove($tree);
 
 Return: HTML::MyHTML::Tree::Node if successful, otherwise a UNDEF value
 
@@ -716,6 +884,87 @@ Print tree of a node. Print including current node
  $node->print_all($tree, $fh);
 
 
+=head2 Token Node
+
+=head3 info
+
+Get information of token node: tag name, tag id, attr
+
+ my $res = $token_node->info($tree);
+
+Return: hash ref
+
+
+=head3 tag_id
+
+Get token node tag id
+
+ my $tag_id = $token_node->tag_id();
+
+Return: tag_id
+
+
+=head3 tag_name
+
+Get tag name of a token node
+
+ my $res = $token_node->tag_name($tree);
+
+Return: tag name
+
+
+=head3 is_close_self
+
+Node has self-closing flag?
+
+ my $bool = $token_node->is_close_self();
+
+Return: 1 (true) or 0 (false)
+
+
+=head3 attr_first
+
+Get first attribute of a token node
+
+ my $attr = $token_node->attr_first();
+
+Return: HTML::MyHTML::Tree::Attr if exists, otherwise an UNDEF value
+
+
+=head3 attr_last
+
+Get last attribute of a token node
+
+ my $attr = $token_node->attr_last();
+
+Return: HTML::MyHTML::Tree::Attr if exists, otherwise an UNDEF value
+
+
+=head3 text
+
+Get text of a token node. Only for a MyHTML_TAG__TEXT or MyHTML_TAG__COMMENT tags
+
+ my $res = $token_node->text();
+
+Return: text if exists, otherwise an UNDEF value
+
+
+=head3 string
+
+Get myhtml_string_t object by token node
+
+ my $string = $token_node->string();
+
+Return: HTML::MyHTML::String if exists, otherwise an NULL value
+
+
+=head3 wait_for_done
+
+Wait for process token all parsing stage. Need if you use thread mode
+
+ $token_node->wait_for_done();
+
+
 =head2 Detect encoding
 
 =head3 encoding_detect
@@ -771,6 +1020,109 @@ Now available for detect UTF-8, UTF-16LE, UTF-16BE
 Return: 1 (true) if encoding found, otherwise 0 (false)
 
 
+=head2 Incoming Buffer
+
+=head3 find_by_position
+
+Get Incoming Buffer by position
+
+ my $incoming_buffer = $incoming_buffer->find_by_position($begin_position);
+
+Return: HTML::Incoming::Buffer if successful, otherwise a UNDEF value
+
+
+=head3 data
+
+Get data of Incoming Buffer
+
+ my $data = $incoming_buffer->data();
+
+Return: text scalar if successful, otherwise a UNDEF value
+
+
+=head3 length
+
+Get data length of Incoming Buffer
+
+ my $length = $incoming_buffer->length();
+
+Return: scalar length
+
+
+=head3 size
+
+Get data size of Incoming Buffer
+
+ my $size = $incoming_buffer->size();
+
+Return: scalar size
+
+
+=head3 offset
+
+Get data offset of Incoming Buffer. Global position of begin Incoming Buffer.
+
+ my $offset = $incoming_buffer->offset();
+
+Return: scalar offset
+
+
+=head3 relative_begin
+
+Get Relative Position for Incoming Buffer. Incoming Buffer should be prepared by find_by_position.
+
+ my $relative_begin = $incoming_buffer->relative_begin();
+
+Return: scalar relative begin
+
+
+=head3 available_length
+
+This function returns number of available data by Incoming Buffer. Incoming buffer may be incomplete. See next.
+
+ my $available_length = $incoming_buffer->available_length();
+
+Return: scalar available length
+
+
+=head3 next
+
+Get next buffer
+
+ my $next_incoming_buffer = $incoming_buffer->next();
+
+Return: HTML::Incoming::Buffer if exists, otherwise a UNDEF value
+
+
+=head3 prev
+
+Get prev buffer
+
+ my $prev_incoming_buffer = $incoming_buffer->prev();
+
+Return: HTML::Incoming::Buffer if exists, otherwise a UNDEF value
+
+
+=head2 Namespace
+
+=head3 namespace_name_by_id
+
+Get namespace text by namespace type (id)
+
+ my $namespace_name = namespace_name_by_id($namespace_id);
+
+Return: text if successful, otherwise a UNDEF value
+
+
+=head3 namespace_id_by_name
+
+Get namespace type (id) by namespace text
+
+ my $namespace_id = namespace_id_by_name($namespace_name);
+
+Return: namespace id
+
+
 =head1 Constants
 
 =head2 Tags
@@ -1030,6 +1382,14 @@ Return: 1 (true) if encoding found, otherwise 0 (false)
  MyHTML_TAG_FIRST_ENTRY
  MyHTML_TAG_LAST_ENTRY
 
+ 
+=head2 Tree parse flags
+
+ MyHTML_TREE_PARSE_FLAGS_CLEAN
+ MyHTML_TREE_PARSE_FLAGS_WITHOUT_BUILD_TREE
+ MyHTML_TREE_PARSE_FLAGS_WITHOUT_PROCESS_TOKEN
+ MyHTML_TREE_PARSE_FLAGS_SKIP_WHITESPACE_TOKEN
+ MyHTML_TREE_PARSE_FLAGS_WITHOUT_DOCTYPE_IN_TREE
 
 =head2 Namespaces
 
@@ -1080,6 +1440,11 @@ Return: 1 (true) if encoding found, otherwise 0 (false)
  MyHTML_ENCODING_WINDOWS_1258
  MyHTML_ENCODING_WINDOWS_874
  MyHTML_ENCODING_X_MAC_CYRILLIC
+ MyHTML_ENCODING_ISO_2022_JP
+ MyHTML_ENCODING_GBK
+ MyHTML_ENCODING_SHIFT_JIS
+ MyHTML_ENCODING_EUC_JP
+ MyHTML_ENCODING_ISO_8859_8_I
  MyHTML_ENCODING_LAST_ENTRY
 
 
@@ -1087,6 +1452,7 @@ Return: 1 (true) if encoding found, otherwise 0 (false)
 
  MyHTML_STATUS_OK
  MyHTML_STATUS_ERROR_MEMORY_ALLOCATION
+ MyHTML_STATUS_THREAD_ERROR_MEMORY_ALLOCATION
  MyHTML_STATUS_THREAD_ERROR_LIST_INIT
  MyHTML_STATUS_THREAD_ERROR_ATTR_MALLOC
  MyHTML_STATUS_THREAD_ERROR_ATTR_INIT
@@ -1103,10 +1469,15 @@ Return: 1 (true) if encoding found, otherwise 0 (false)
  MyHTML_STATUS_THREAD_ERROR_QUEUE_MALLOC
  MyHTML_STATUS_THREAD_ERROR_QUEUE_NODES_MALLOC
  MyHTML_STATUS_THREAD_ERROR_QUEUE_NODE_MALLOC
+ MyHTML_STATUS_THREAD_ERROR_MUTEX_MALLOC
+ MyHTML_STATUS_THREAD_ERROR_MUTEX_INIT
+ MyHTML_STATUS_THREAD_ERROR_MUTEX_LOCK
+ MyHTML_STATUS_THREAD_ERROR_MUTEX_UNLOCK
  MyHTML_STATUS_RULES_ERROR_MEMORY_ALLOCATION
  MyHTML_STATUS_PERF_ERROR_COMPILED_WITHOUT_PERF
  MyHTML_STATUS_PERF_ERROR_FIND_CPU_CLOCK
  MyHTML_STATUS_TOKENIZER_ERROR_MEMORY_ALLOCATION
+ MyHTML_STATUS_TOKENIZER_ERROR_FRAGMENT_INIT
  MyHTML_STATUS_TAGS_ERROR_MEMORY_ALLOCATION
  MyHTML_STATUS_TAGS_ERROR_MCOBJECT_CREATE
  MyHTML_STATUS_TAGS_ERROR_MCOBJECT_MALLOC
@@ -1117,8 +1488,18 @@ Return: 1 (true) if encoding found, otherwise 0 (false)
  MyHTML_STATUS_TREE_ERROR_MCOBJECT_CREATE
  MyHTML_STATUS_TREE_ERROR_MCOBJECT_INIT
  MyHTML_STATUS_TREE_ERROR_MCOBJECT_CREATE_NODE
+ MyHTML_STATUS_TREE_ERROR_INCOMING_BUFFER_CREATE
  MyHTML_STATUS_ATTR_ERROR_ALLOCATION
  MyHTML_STATUS_ATTR_ERROR_CREATE
+ MyHTML_STATUS_STREAM_BUFFER_ERROR_CREATE
+ MyHTML_STATUS_STREAM_BUFFER_ERROR_INIT
+ MyHTML_STATUS_STREAM_BUFFER_ENTRY_ERROR_CREATE
+ MyHTML_STATUS_STREAM_BUFFER_ENTRY_ERROR_INIT
+ MyHTML_STATUS_STREAM_BUFFER_ERROR_ADD_ENTRY
+ MyHTML_STATUS_MCOBJECT_ERROR_CACHE_CREATE
+ MyHTML_STATUS_MCOBJECT_ERROR_CHUNK_CREATE
+ MyHTML_STATUS_MCOBJECT_ERROR_CHUNK_INIT
+ MyHTML_STATUS_MCOBJECT_ERROR_CACHE_REALLOC
 
 
 =head2 Options
@@ -1127,10 +1508,6 @@ Return: 1 (true) if encoding found, otherwise 0 (false)
  MyHTML_OPTIONS_PARSE_MODE_SINGLE
  MyHTML_OPTIONS_PARSE_MODE_ALL_IN_ONE
  MyHTML_OPTIONS_PARSE_MODE_SEPARATELY
- MyHTML_OPTIONS_PARSE_MODE_WORKER_TREE
- MyHTML_OPTIONS_PARSE_MODE_WORKER_INDEX
- MyHTML_OPTIONS_PARSE_MODE_TREE_INDEX
-
 
 =head1 Examples
 
