@@ -1,17 +1,19 @@
 /*
- Copyright 2015-2016 Alexander Borisov
+ Copyright (C) 2015-2016 Alexander Borisov
  
- Licensed under the Apache License, Version 2.0 (the "License");
- you may not use this file except in compliance with the License.
- You may obtain a copy of the License at
+ This library is free software; you can redistribute it and/or
+ modify it under the terms of the GNU Lesser General Public
+ License as published by the Free Software Foundation; either
+ version 2.1 of the License, or (at your option) any later version.
  
- http://www.apache.org/licenses/LICENSE-2.0
+ This library is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+ Lesser General Public License for more details.
  
- Unless required by applicable law or agreed to in writing, software
- distributed under the License is distributed on an "AS IS" BASIS,
- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- See the License for the specific language governing permissions and
- limitations under the License.
+ You should have received a copy of the GNU Lesser General Public
+ License along with this library; if not, write to the Free Software
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  
  Author: lex.borisov@gmail.com (Alexander Borisov)
 */
@@ -27,8 +29,11 @@
 #include <stdint.h>
 #include <stdarg.h>
 
+#define MyHTML_VERSION_MAJOR 1
+#define MyHTML_VERSION_MINOR 0
+#define MyHTML_VERSION_PATCH 2
 
-#if defined(_WIN32) || defined(_WIN64)
+#if (defined(_WIN32) || defined(_WIN64)) && !defined(__WINPTHREADS_VERSION)
 #define IS_OS_WINDOWS
 #include <windows.h>
 #endif
@@ -45,26 +50,56 @@
 extern "C" {
 #endif
 
-#ifdef DEBUG_MODE
-    #define MyHTML_DEBUG(__format__, ...)      \
-        myhtml_print(stderr, "DEBUG: "__format__"\n", ##__VA_ARGS__)
+/* Mem */
+#ifdef MyHTML_EXTERN_MALLOC
+    extern void * MyHTML_EXTERN_MALLOC(size_t size);
+    #define myhtml_malloc MyHTML_EXTERN_MALLOC
 #else
-    #define MyHTML_DEBUG(__format__, ...)
+    #define myhtml_malloc myhtml_mem_malloc
+#endif
+
+#ifdef MyHTML_EXTERN_REALLOC
+    extern void * MyHTML_EXTERN_REALLOC(void* dst, size_t size);
+    #define myhtml_realloc MyHTML_EXTERN_REALLOC
+#else
+    #define myhtml_realloc myhtml_mem_realloc
+#endif
+
+#ifdef MyHTML_EXTERN_CALLOC
+    extern void * MyHTML_EXTERN_CALLOC(size_t num, size_t size);
+    #define myhtml_calloc MyHTML_EXTERN_CALLOC
+#else
+    #define myhtml_calloc myhtml_mem_calloc
+#endif
+
+#ifdef MyHTML_EXTERN_FREE
+    extern void MyHTML_EXTERN_FREE(void* dst);
+    #define myhtml_free MyHTML_EXTERN_FREE
+#else
+    #define myhtml_free myhtml_mem_free
+#endif
+
+/* Debug */
+#ifdef DEBUG_MODE
+    #define MyHTML_DEBUG(format, ...)      \
+        myhtml_print(stderr, "DEBUG: "format"\n", ##__VA_ARGS__)
+#else
+    #define MyHTML_DEBUG(format, ...)
 #endif
 
 #ifdef DEBUG_MODE
-    #define MyHTML_DEBUG_ERROR(__format__, ...)      \
-        myhtml_print(stderr, "DEBUG ERROR: "__format__"\n", ##__VA_ARGS__)
+    #define MyHTML_DEBUG_ERROR(format, ...)      \
+        myhtml_print(stderr, "DEBUG ERROR: "format"\n", ##__VA_ARGS__)
 #else
-    #define MyHTML_DEBUG_ERROR(__format__, ...)
+    #define MyHTML_DEBUG_ERROR(format, ...)
 #endif
 
-#define myhtml_base_add(__myhtml__, __point__, __lenn__, __sizen__, __strcn__, __size__)    \
-    __myhtml__->__lenn__++;                                                                 \
-    if(__myhtml__->__lenn__ == __myhtml__->__sizen__) {                                     \
-        __myhtml__->__sizen__ += __size__;                                                  \
-        __myhtml__->__point__ = (__strcn__*)myrealloc(__myhtml__->__point__,                \
-            sizeof(__strcn__) * __myhtml__->__sizen__);                                     \
+#define myhtml_base_add(myhtml, point, lenn, sizen, strcn, size)    \
+    myhtml->Lenn++;                                                 \
+    if(myhtml->lenn == myhtml->sizen) {                             \
+        myhtml->sizen += size;                                      \
+        myhtml->point = (strcn*)myhtml_realloc(myhtml->point,            \
+            sizeof(strcn) * myhtml->sizen);                         \
     }
 
 
@@ -115,10 +150,18 @@ enum myhtml_encoding_list {
     MyHTML_ENCODING_WINDOWS_1258     = 0x22,
     MyHTML_ENCODING_WINDOWS_874      = 0x23,
     MyHTML_ENCODING_X_MAC_CYRILLIC   = 0x24,
-    MyHTML_ENCODING_LAST_ENTRY       = 0x25
+    MyHTML_ENCODING_ISO_2022_JP      = 0x25,
+    MyHTML_ENCODING_GBK              = 0x26,
+    MyHTML_ENCODING_SHIFT_JIS        = 0x27,
+    MyHTML_ENCODING_EUC_JP           = 0x28,
+    MyHTML_ENCODING_ISO_8859_8_I     = 0x29,
+    MyHTML_ENCODING_LAST_ENTRY       = 0x2a
 }
 typedef myhtml_encoding_t;
 
+// char references
+typedef struct myhtml_data_process_entry myhtml_data_process_entry_t;
+
 // strings
 typedef struct myhtml_string myhtml_string_t;
 
@@ -156,7 +199,15 @@ enum myhtml_tree_flags {
     MyHTML_TREE_FLAGS_PARSE_FLAG_EMIT_NEWLINE = 0x080
 };
 
-typedef struct myhtml_tree_temp_stream myhtml_tree_temp_stream_t;
+enum myhtml_tree_parse_flags {
+    MyHTML_TREE_PARSE_FLAGS_CLEAN                   = 0x000,
+    MyHTML_TREE_PARSE_FLAGS_WITHOUT_BUILD_TREE      = 0x001,
+    MyHTML_TREE_PARSE_FLAGS_WITHOUT_PROCESS_TOKEN   = 0x003,
+    MyHTML_TREE_PARSE_FLAGS_SKIP_WHITESPACE_TOKEN   = 0x004,
+    MyHTML_TREE_PARSE_FLAGS_WITHOUT_DOCTYPE_IN_TREE = 0x008,
+}
+typedef myhtml_tree_parse_flags_t;
+
 typedef struct myhtml_tree_temp_tag_name myhtml_tree_temp_tag_name_t;
 typedef struct myhtml_tree_insertion_list myhtml_tree_insertion_list_t;
 typedef struct myhtml_tree_token_list myhtml_tree_token_list_t;
@@ -215,6 +266,10 @@ typedef size_t myhtml_tag_id_t;
 
 typedef struct myhtml_tag myhtml_tag_t;
 
+// stream
+typedef struct myhtml_stream_buffer_entry myhtml_stream_buffer_entry_t;
+typedef struct myhtml_stream_buffer myhtml_stream_buffer_t;
+
 // parse
 enum myhtml_tokenizer_state {
     MyHTML_TOKENIZER_STATE_DATA                                          = 0x000,
@@ -318,45 +373,68 @@ enum myhtml_insertion_mode {
 };
 
 // base
+/*
+ Very important!!!
+ 
+ for myhtml             0..00ffff;      MyHTML_STATUS_OK    == 0x000000
+ for mycss and modules  010000..01ffff; MyCSS_STATUS_OK     == 0x000000
+ for myrender           020000..02ffff; MyRENDER_STATUS_OK  == 0x000000
+ for mydom              030000..03ffff; MyDOM_STATUS_OK     == 0x000000
+ for mynetwork          040000..04ffff; MyNETWORK_STATUS_OK == 0x000000
+ for myecma             050000..05ffff; MyECMA_STATUS_OK    == 0x000000
+ not occupied           060000..
+*/
 enum myhtml_status {
-    MyHTML_STATUS_OK                                   = 0,
-    MyHTML_STATUS_ERROR_MEMORY_ALLOCATION              = 1,
-    MyHTML_STATUS_THREAD_ERROR_LIST_INIT               = 10,
-    MyHTML_STATUS_THREAD_ERROR_ATTR_MALLOC             = 11,
-    MyHTML_STATUS_THREAD_ERROR_ATTR_INIT               = 12,
-    MyHTML_STATUS_THREAD_ERROR_ATTR_SET                = 13,
-    MyHTML_STATUS_THREAD_ERROR_ATTR_DESTROY            = 14,
-    MyHTML_STATUS_THREAD_ERROR_NO_SLOTS                = 15,
-    MyHTML_STATUS_THREAD_ERROR_BATCH_INIT              = 16,
-    MyHTML_STATUS_THREAD_ERROR_WORKER_MALLOC           = 17,
-    MyHTML_STATUS_THREAD_ERROR_WORKER_SEM_CREATE       = 18,
-    MyHTML_STATUS_THREAD_ERROR_WORKER_THREAD_CREATE    = 19,
-    MyHTML_STATUS_THREAD_ERROR_MASTER_THREAD_CREATE    = 20,
-    MyHTML_STATUS_THREAD_ERROR_SEM_PREFIX_MALLOC       = 50,
-    MyHTML_STATUS_THREAD_ERROR_SEM_CREATE              = 51,
-    MyHTML_STATUS_THREAD_ERROR_QUEUE_MALLOC            = 60,
-    MyHTML_STATUS_THREAD_ERROR_QUEUE_NODES_MALLOC      = 61,
-    MyHTML_STATUS_THREAD_ERROR_QUEUE_NODE_MALLOC       = 62,
-    MyHTML_STATUS_THREAD_ERROR_MUTEX_MALLOC            = 70,
-    MyHTML_STATUS_THREAD_ERROR_MUTEX_INIT              = 71,
-    MyHTML_STATUS_THREAD_ERROR_MUTEX_LOCK              = 72,
-    MyHTML_STATUS_THREAD_ERROR_MUTEX_UNLOCK            = 73,
-    MyHTML_STATUS_RULES_ERROR_MEMORY_ALLOCATION        = 100,
-    MyHTML_STATUS_PERF_ERROR_COMPILED_WITHOUT_PERF     = 200,
-    MyHTML_STATUS_PERF_ERROR_FIND_CPU_CLOCK            = 201,
-    MyHTML_STATUS_TOKENIZER_ERROR_MEMORY_ALLOCATION    = 300,
-    MyHTML_STATUS_TAGS_ERROR_MEMORY_ALLOCATION         = 400,
-    MyHTML_STATUS_TAGS_ERROR_MCOBJECT_CREATE           = 401,
-    MyHTML_STATUS_TAGS_ERROR_MCOBJECT_MALLOC           = 402,
-    MyHTML_STATUS_TAGS_ERROR_MCOBJECT_CREATE_NODE      = 403,
-    MyHTML_STATUS_TAGS_ERROR_CACHE_MEMORY_ALLOCATION   = 404,
-    MyHTML_STATUS_TAGS_ERROR_INDEX_MEMORY_ALLOCATION   = 405,
-    MyHTML_STATUS_TREE_ERROR_MEMORY_ALLOCATION         = 500,
-    MyHTML_STATUS_TREE_ERROR_MCOBJECT_CREATE           = 501,
-    MyHTML_STATUS_TREE_ERROR_MCOBJECT_INIT             = 502,
-    MyHTML_STATUS_TREE_ERROR_MCOBJECT_CREATE_NODE      = 503,
-    MyHTML_STATUS_ATTR_ERROR_ALLOCATION                = 600,
-    MyHTML_STATUS_ATTR_ERROR_CREATE                    = 601
+    MyHTML_STATUS_OK                                   = 0x0000,
+    MyHTML_STATUS_ERROR_MEMORY_ALLOCATION              = 0x0001,
+    MyHTML_STATUS_THREAD_ERROR_MEMORY_ALLOCATION       = 0x0009,
+    MyHTML_STATUS_THREAD_ERROR_LIST_INIT               = 0x000a,
+    MyHTML_STATUS_THREAD_ERROR_ATTR_MALLOC             = 0x000b,
+    MyHTML_STATUS_THREAD_ERROR_ATTR_INIT               = 0x000c,
+    MyHTML_STATUS_THREAD_ERROR_ATTR_SET                = 0x000d,
+    MyHTML_STATUS_THREAD_ERROR_ATTR_DESTROY            = 0x000e,
+    MyHTML_STATUS_THREAD_ERROR_NO_SLOTS                = 0x000f,
+    MyHTML_STATUS_THREAD_ERROR_BATCH_INIT              = 0x0010,
+    MyHTML_STATUS_THREAD_ERROR_WORKER_MALLOC           = 0x0011,
+    MyHTML_STATUS_THREAD_ERROR_WORKER_SEM_CREATE       = 0x0012,
+    MyHTML_STATUS_THREAD_ERROR_WORKER_THREAD_CREATE    = 0x0013,
+    MyHTML_STATUS_THREAD_ERROR_MASTER_THREAD_CREATE    = 0x0014,
+    MyHTML_STATUS_THREAD_ERROR_SEM_PREFIX_MALLOC       = 0x0032,
+    MyHTML_STATUS_THREAD_ERROR_SEM_CREATE              = 0x0033,
+    MyHTML_STATUS_THREAD_ERROR_QUEUE_MALLOC            = 0x003c,
+    MyHTML_STATUS_THREAD_ERROR_QUEUE_NODES_MALLOC      = 0x003d,
+    MyHTML_STATUS_THREAD_ERROR_QUEUE_NODE_MALLOC       = 0x003e,
+    MyHTML_STATUS_THREAD_ERROR_MUTEX_MALLOC            = 0x0046,
+    MyHTML_STATUS_THREAD_ERROR_MUTEX_INIT              = 0x0047,
+    MyHTML_STATUS_THREAD_ERROR_MUTEX_LOCK              = 0x0048,
+    MyHTML_STATUS_THREAD_ERROR_MUTEX_UNLOCK            = 0x0049,
+    MyHTML_STATUS_RULES_ERROR_MEMORY_ALLOCATION        = 0x0064,
+    MyHTML_STATUS_PERF_ERROR_COMPILED_WITHOUT_PERF     = 0x00c8,
+    MyHTML_STATUS_PERF_ERROR_FIND_CPU_CLOCK            = 0x00c9,
+    MyHTML_STATUS_TOKENIZER_ERROR_MEMORY_ALLOCATION    = 0x012c,
+    MyHTML_STATUS_TOKENIZER_ERROR_FRAGMENT_INIT        = 0x012d,
+    MyHTML_STATUS_TAGS_ERROR_MEMORY_ALLOCATION         = 0x0190,
+    MyHTML_STATUS_TAGS_ERROR_MCOBJECT_CREATE           = 0x0191,
+    MyHTML_STATUS_TAGS_ERROR_MCOBJECT_MALLOC           = 0x0192,
+    MyHTML_STATUS_TAGS_ERROR_MCOBJECT_CREATE_NODE      = 0x0193,
+    MyHTML_STATUS_TAGS_ERROR_CACHE_MEMORY_ALLOCATION   = 0x0194,
+    MyHTML_STATUS_TAGS_ERROR_INDEX_MEMORY_ALLOCATION   = 0x0195,
+    MyHTML_STATUS_TREE_ERROR_MEMORY_ALLOCATION         = 0x01f4,
+    MyHTML_STATUS_TREE_ERROR_MCOBJECT_CREATE           = 0x01f5,
+    MyHTML_STATUS_TREE_ERROR_MCOBJECT_INIT             = 0x01f6,
+    MyHTML_STATUS_TREE_ERROR_MCOBJECT_CREATE_NODE      = 0x01f7,
+    MyHTML_STATUS_TREE_ERROR_INCOMING_BUFFER_CREATE    = 0x01f8,
+    MyHTML_STATUS_ATTR_ERROR_ALLOCATION                = 0x0258,
+    MyHTML_STATUS_ATTR_ERROR_CREATE                    = 0x0259,
+    MyHTML_STATUS_STREAM_BUFFER_ERROR_CREATE           = 0x0300,
+    MyHTML_STATUS_STREAM_BUFFER_ERROR_INIT             = 0x0301,
+    MyHTML_STATUS_STREAM_BUFFER_ENTRY_ERROR_CREATE     = 0x0302,
+    MyHTML_STATUS_STREAM_BUFFER_ENTRY_ERROR_INIT       = 0x0303,
+    MyHTML_STATUS_STREAM_BUFFER_ERROR_ADD_ENTRY        = 0x0304,
+    MyHTML_STATUS_MCOBJECT_ERROR_CACHE_CREATE          = 0x0340,
+    MyHTML_STATUS_MCOBJECT_ERROR_CHUNK_CREATE          = 0x0341,
+    MyHTML_STATUS_MCOBJECT_ERROR_CHUNK_INIT            = 0x0342,
+    MyHTML_STATUS_MCOBJECT_ERROR_CACHE_REALLOC         = 0x0343
 }
 typedef myhtml_status_t;
 
@@ -376,19 +454,29 @@ enum myhtml_options {
     MyHTML_OPTIONS_DEFAULT                 = 0x00,
     MyHTML_OPTIONS_PARSE_MODE_SINGLE       = 0x01,
     MyHTML_OPTIONS_PARSE_MODE_ALL_IN_ONE   = 0x02,
-    MyHTML_OPTIONS_PARSE_MODE_SEPARATELY   = 0x04,
-    MyHTML_OPTIONS_PARSE_MODE_WORKER_TREE  = 0x08,
-    MyHTML_OPTIONS_PARSE_MODE_WORKER_INDEX = 0x10,
-    MyHTML_OPTIONS_PARSE_MODE_TREE_INDEX   = 0x20
+    MyHTML_OPTIONS_PARSE_MODE_SEPARATELY   = 0x04
 };
 
-typedef struct myhtml_incoming_buf myhtml_incoming_buf_t;
+struct myhtml_position {
+    size_t begin;
+    size_t length;
+}
+typedef myhtml_position_t;
+
+struct myhtml_version {
+    int major;
+    int minor;
+    int patch;
+}
+typedef myhtml_version_t;
+
+typedef struct myhtml_incoming_buffer myhtml_incoming_buffer_t;
 typedef myhtml_token_attr_t myhtml_tree_attr_t;
 typedef struct myhtml_collection myhtml_collection_t;
 typedef struct myhtml myhtml_t;
 
 // parser state function
-typedef size_t (*myhtml_tokenizer_state_f)(myhtml_tree_t* tree, mythread_queue_node_t* qnode, const char* html, size_t html_offset, size_t html_size);
+typedef size_t (*myhtml_tokenizer_state_f)(myhtml_tree_t* tree, myhtml_token_node_t* token_node, const char* html, size_t html_offset, size_t html_size);
 
 // parser stream function
 typedef void (*mythread_f)(mythread_id_t thread_id, mythread_queue_node_t *qnode);
@@ -396,10 +484,21 @@ typedef void (*mythread_f)(mythread_id_t thread_id, mythread_queue_node_t *qnode
 // parser insertion mode function
 typedef bool (*myhtml_insertion_f)(myhtml_tree_t* tree, myhtml_token_node_t* token);
 
-void * mymalloc(size_t size);
-void * myrealloc(void* dst, size_t size);
-void * mycalloc(size_t num, size_t size);
-void   myfree(void* dst);
+// char references state
+typedef size_t (*myhtml_data_process_state_f)(myhtml_data_process_entry_t* charef, myhtml_string_t* str, const char* data, size_t offset, size_t size);
+
+// callback functions
+typedef void* (*myhtml_callback_token_f)(myhtml_tree_t* tree, myhtml_token_node_t* token, void* ctx);
+typedef void (*myhtml_callback_tree_node_f)(myhtml_tree_t* tree, myhtml_tree_node_t* node, void* ctx);
+
+// find attribute value functions
+typedef bool (*myhtml_attribute_value_find_f)(myhtml_string_t* str_key, const char* value, size_t value_len);
+
+
+void * myhtml_mem_malloc(size_t size);
+void * myhtml_mem_realloc(void* dst, size_t size);
+void * myhtml_mem_calloc(size_t num, size_t size);
+void   myhtml_mem_free(void* dst);
 
 void myhtml_print(FILE* out, const char* format, ...);
 
