@@ -1,17 +1,19 @@
 /*
- Copyright 2015-2016 Alexander Borisov
+ Copyright (C) 2015-2016 Alexander Borisov
  
- Licensed under the Apache License, Version 2.0 (the "License");
- you may not use this file except in compliance with the License.
- You may obtain a copy of the License at
+ This library is free software; you can redistribute it and/or
+ modify it under the terms of the GNU Lesser General Public
+ License as published by the Free Software Foundation; either
+ version 2.1 of the License, or (at your option) any later version.
  
- http://www.apache.org/licenses/LICENSE-2.0
+ This library is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+ Lesser General Public License for more details.
  
- Unless required by applicable law or agreed to in writing, software
- distributed under the License is distributed on an "AS IS" BASIS,
- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- See the License for the specific language governing permissions and
- limitations under the License.
+ You should have received a copy of the GNU Lesser General Public
+ License along with this library; if not, write to the Free Software
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  
  Author: lex.borisov@gmail.com (Alexander Borisov)
 */
@@ -136,7 +138,7 @@ myhtml_status_t myhtml_thread_cancel(mythread_t *mythread, mythread_list_t *thr)
 
 myhtml_status_t myhtml_thread_attr_init(mythread_t *mythread)
 {
-    mythread->attr = (pthread_attr_t*)calloc(1, sizeof(pthread_attr_t));
+    mythread->attr = (pthread_attr_t*)myhtml_calloc(1, sizeof(pthread_attr_t));
     
     if(mythread->attr == NULL)
         return MyHTML_STATUS_THREAD_ERROR_ATTR_MALLOC;
@@ -163,7 +165,7 @@ myhtml_status_t myhtml_thread_attr_destroy(mythread_t *mythread)
     if(mythread->attr) {
         mythread->sys_last_error = pthread_attr_destroy(mythread->attr);
         
-        free(mythread->attr);
+        myhtml_free(mythread->attr);
         mythread->attr = NULL;
         
         if(mythread->sys_last_error)
@@ -175,7 +177,7 @@ myhtml_status_t myhtml_thread_attr_destroy(mythread_t *mythread)
 
 myhtml_status_t myhtml_hread_mutex_create(mythread_t *mythread, mythread_context_t *ctx, size_t prefix_id)
 {
-    ctx->mutex = (pthread_mutex_t*)calloc(1, sizeof(pthread_mutex_t));
+    ctx->mutex = (pthread_mutex_t*)myhtml_calloc(1, sizeof(pthread_mutex_t));
     
     if(ctx->mutex == NULL)
         return MyHTML_STATUS_THREAD_ERROR_MUTEX_MALLOC;
@@ -227,7 +229,7 @@ myhtml_status_t myhtml_hread_mutex_close(mythread_t *mythread, mythread_context_
 {
     if(ctx->mutex) {
         pthread_mutex_destroy(ctx->mutex);
-        free(ctx->mutex);
+        myhtml_free(ctx->mutex);
         
         ctx->mutex = NULL;
     }
@@ -251,7 +253,7 @@ void myhtml_thread_nanosleep(const struct timespec *tomeout)
 
 mythread_t * mythread_create(void)
 {
-    return calloc(1, sizeof(mythread_t));
+    return myhtml_calloc(1, sizeof(mythread_t));
 }
 
 #ifdef MyHTML_BUILD_WITHOUT_THREADS
@@ -279,7 +281,7 @@ myhtml_status_t mythread_init(mythread_t *mythread, const char *sem_prefix, size
         mythread->pth_list_root   = 1;
         mythread->pth_list_length = 1;
         mythread->pth_list_size   = thread_count + 1;
-        mythread->pth_list        = (mythread_list_t*)calloc(mythread->pth_list_size, sizeof(mythread_list_t));
+        mythread->pth_list        = (mythread_list_t*)myhtml_calloc(mythread->pth_list_size, sizeof(mythread_list_t));
         
         if(mythread->pth_list == NULL)
             return MyHTML_STATUS_THREAD_ERROR_LIST_INIT;
@@ -295,7 +297,7 @@ myhtml_status_t mythread_init(mythread_t *mythread, const char *sem_prefix, size
     }
     
     myhtml_status_t status;
-    mythread->queue_list = mythread_queue_list_create(mythread, 1024, &status);
+    mythread->queue_list = mythread_queue_list_create(mythread, &status);
     
     if(mythread->queue_list == NULL)
         return status;
@@ -305,7 +307,7 @@ myhtml_status_t mythread_init(mythread_t *mythread, const char *sem_prefix, size
         mythread->sem_prefix_length = strlen(sem_prefix);
         
         if(mythread->sem_prefix_length) {
-            mythread->sem_prefix = calloc((mythread->sem_prefix_length + 1), sizeof(char));
+            mythread->sem_prefix = myhtml_calloc((mythread->sem_prefix_length + 1), sizeof(char));
             
             if(mythread->sem_prefix == NULL) {
                 mythread->sem_prefix_length = 0;
@@ -346,16 +348,16 @@ mythread_t * mythread_destroy(mythread_t *mythread, bool self_destroy)
             myhtml_thread_join(mythread, &mythread->pth_list[i]);
         }
         
-        free(mythread->pth_list);
+        myhtml_free(mythread->pth_list);
         mythread->pth_list = NULL;
     }
     
     if(mythread->queue_list) {
-        free(mythread->queue_list);
+        myhtml_free(mythread->queue_list);
     }
     
     if(mythread->sem_prefix) {
-        free(mythread->sem_prefix);
+        myhtml_free(mythread->sem_prefix);
         
         mythread->sem_prefix = NULL;
         mythread->sem_prefix_length = 0;
@@ -364,7 +366,7 @@ mythread_t * mythread_destroy(mythread_t *mythread, bool self_destroy)
 #endif /* MyHTML_BUILD_WITHOUT_THREADS */
     
     if(self_destroy) {
-        free(mythread);
+        myhtml_free(mythread);
         return NULL;
     }
     
@@ -473,16 +475,14 @@ mythread_id_t myhread_create_batch(mythread_t *mythread, mythread_f func, myhtml
 #endif /* MyHTML_BUILD_WITHOUT_THREADS */
 
 // mythread queue functions
+// TODO: size = 1024; -- never use
 #ifndef MyHTML_BUILD_WITHOUT_THREADS
-mythread_queue_list_t * mythread_queue_list_create(mythread_t *mythread, size_t size, myhtml_status_t *status)
+mythread_queue_list_t * mythread_queue_list_create(mythread_t *mythread, myhtml_status_t *status)
 {
     if(status)
         *status = MyHTML_STATUS_OK;
     
-    if(size < 1024)
-        size = 1024;
-    
-    mythread_queue_list_t* queue_list = (mythread_queue_list_t*)mycalloc(1, sizeof(mythread_queue_list_t));
+    mythread_queue_list_t* queue_list = (mythread_queue_list_t*)myhtml_calloc(1, sizeof(mythread_queue_list_t));
     
     if(queue_list == NULL) {
         if(status)
@@ -500,7 +500,7 @@ mythread_queue_list_entry_t * mythread_queue_list_entry_push(mythread_t *mythrea
     if(status)
         *status = MyHTML_STATUS_OK;
     
-    mythread_queue_list_entry_t* entry = (mythread_queue_list_entry_t*)mycalloc(1, sizeof(mythread_queue_list_entry_t));
+    mythread_queue_list_entry_t* entry = (mythread_queue_list_entry_t*)myhtml_calloc(1, sizeof(mythread_queue_list_entry_t));
     
     if(entry == NULL) {
         if(status)
@@ -508,10 +508,10 @@ mythread_queue_list_entry_t * mythread_queue_list_entry_push(mythread_t *mythrea
         return NULL;
     }
     
-    entry->thread_param = (mythread_queue_thread_param_t*)mycalloc(mythread->pth_list_size, sizeof(mythread_queue_thread_param_t));
+    entry->thread_param = (mythread_queue_thread_param_t*)myhtml_calloc(mythread->pth_list_size, sizeof(mythread_queue_thread_param_t));
     
-    if(entry == NULL) {
-        free(entry);
+    if(entry->thread_param == NULL) {
+        myhtml_free(entry);
         
         if(status)
             *status = MyHTML_STATUS_THREAD_ERROR_QUEUE_MALLOC;
@@ -584,9 +584,9 @@ mythread_queue_list_entry_t * mythread_queue_list_entry_delete(mythread_t *mythr
         mythread_queue_destroy(entry->queue);
     
     if(entry->thread_param)
-        free(entry->thread_param);
+        myhtml_free(entry->thread_param);
     
-    free(entry);
+    myhtml_free(entry);
     
     queue_list->count--;
     
@@ -636,7 +636,7 @@ mythread_queue_t * mythread_queue_create(size_t size, myhtml_status_t *status)
     if(size < 4096)
         size = 4096;
     
-    mythread_queue_t* queue = (mythread_queue_t*)mymalloc(sizeof(mythread_queue_t));
+    mythread_queue_t* queue = (mythread_queue_t*)myhtml_malloc(sizeof(mythread_queue_t));
     
     if(queue == NULL) {
         if(status)
@@ -646,10 +646,10 @@ mythread_queue_t * mythread_queue_create(size_t size, myhtml_status_t *status)
     
     queue->nodes_pos_size = 512;
     queue->nodes_size     = size;
-    queue->nodes          = (mythread_queue_node_t**)mycalloc(queue->nodes_pos_size, sizeof(mythread_queue_node_t*));
+    queue->nodes          = (mythread_queue_node_t**)myhtml_calloc(queue->nodes_pos_size, sizeof(mythread_queue_node_t*));
     
     if(queue->nodes == NULL) {
-        free(queue);
+        myhtml_free(queue);
         
         if(status)
             *status = MyHTML_STATUS_THREAD_ERROR_QUEUE_NODES_MALLOC;
@@ -658,11 +658,11 @@ mythread_queue_t * mythread_queue_create(size_t size, myhtml_status_t *status)
     
     mythread_queue_clean(queue);
     
-    queue->nodes[queue->nodes_pos] = (mythread_queue_node_t*)mymalloc(sizeof(mythread_queue_node_t) * queue->nodes_size);
+    queue->nodes[queue->nodes_pos] = (mythread_queue_node_t*)myhtml_malloc(sizeof(mythread_queue_node_t) * queue->nodes_size);
     
     if(queue->nodes[queue->nodes_pos] == NULL) {
-        free(queue->nodes);
-        free(queue);
+        myhtml_free(queue->nodes);
+        myhtml_free(queue);
         
         if(status)
             *status = MyHTML_STATUS_THREAD_ERROR_QUEUE_NODE_MALLOC;
@@ -690,13 +690,13 @@ mythread_queue_t * mythread_queue_destroy(mythread_queue_t* queue)
     
     if(queue->nodes) {
         for (size_t i = 0; i <= queue->nodes_pos; i++) {
-            free(queue->nodes[i]);
+            myhtml_free(queue->nodes[i]);
         }
         
-        free(queue->nodes);
+        myhtml_free(queue->nodes);
     }
     
-    free(queue);
+    myhtml_free(queue);
     
     return NULL;
 }
@@ -726,7 +726,7 @@ size_t mythread_queue_count_used_node(mythread_queue_t* queue)
     return queue->nodes_uses;
 }
 
-mythread_queue_node_t * mythread_queue_node_malloc(mythread_t *mythread, mythread_queue_t* queue, const char* text, size_t begin, myhtml_status_t *status)
+mythread_queue_node_t * mythread_queue_node_malloc(mythread_t *mythread, mythread_queue_t* queue, myhtml_status_t *status)
 {
     queue->nodes_length++;
     
@@ -739,7 +739,7 @@ mythread_queue_node_t * mythread_queue_node_malloc(mythread_t *mythread, mythrea
             mythread_wait_all_for_done(mythread);
             
             queue->nodes_pos_size <<= 1;
-            mythread_queue_node_t** tmp = realloc(queue->nodes, sizeof(mythread_queue_node_t*) * queue->nodes_pos_size);
+            mythread_queue_node_t** tmp = myhtml_realloc(queue->nodes, sizeof(mythread_queue_node_t*) * queue->nodes_pos_size);
             
             if(tmp) {
                 memset(&tmp[queue->nodes_pos], 0, sizeof(mythread_queue_node_t*) * (queue->nodes_pos_size - queue->nodes_pos));
@@ -755,7 +755,7 @@ mythread_queue_node_t * mythread_queue_node_malloc(mythread_t *mythread, mythrea
         }
         
         if(queue->nodes[queue->nodes_pos] == NULL) {
-            queue->nodes[queue->nodes_pos] = (mythread_queue_node_t*)malloc(sizeof(mythread_queue_node_t) * queue->nodes_size);
+            queue->nodes[queue->nodes_pos] = (mythread_queue_node_t*)myhtml_malloc(sizeof(mythread_queue_node_t) * queue->nodes_size);
             
             if(queue->nodes[queue->nodes_pos] == NULL) {
                 if(status)
@@ -770,15 +770,10 @@ mythread_queue_node_t * mythread_queue_node_malloc(mythread_t *mythread, mythrea
     
     queue->nodes_uses++;
     
-    mythread_queue_node_t *qnode = &queue->nodes[queue->nodes_pos][queue->nodes_length];
-    
-    qnode->text  = text;
-    qnode->begin = begin;
-    
-    return qnode;
+    return &queue->nodes[queue->nodes_pos][queue->nodes_length];
 }
 
-mythread_queue_node_t * mythread_queue_node_malloc_limit(mythread_t *mythread, mythread_queue_t* queue, const char* text, size_t begin, size_t limit, myhtml_status_t *status)
+mythread_queue_node_t * mythread_queue_node_malloc_limit(mythread_t *mythread, mythread_queue_t* queue, size_t limit, myhtml_status_t *status)
 {
     queue->nodes_length++;
     
@@ -800,7 +795,7 @@ mythread_queue_node_t * mythread_queue_node_malloc_limit(mythread_t *mythread, m
             mythread_wait_all_for_done(mythread);
             
             queue->nodes_pos_size <<= 1;
-            mythread_queue_node_t** tmp = realloc(queue->nodes, sizeof(mythread_queue_node_t*) * queue->nodes_pos_size);
+            mythread_queue_node_t** tmp = myhtml_realloc(queue->nodes, sizeof(mythread_queue_node_t*) * queue->nodes_pos_size);
             
             if(tmp) {
                 memset(&tmp[queue->nodes_pos], 0, sizeof(mythread_queue_node_t*) * (queue->nodes_pos_size - queue->nodes_pos));
@@ -816,7 +811,7 @@ mythread_queue_node_t * mythread_queue_node_malloc_limit(mythread_t *mythread, m
         }
         
         if(queue->nodes[queue->nodes_pos] == NULL) {
-            queue->nodes[queue->nodes_pos] = (mythread_queue_node_t*)malloc(sizeof(mythread_queue_node_t) * queue->nodes_size);
+            queue->nodes[queue->nodes_pos] = (mythread_queue_node_t*)myhtml_malloc(sizeof(mythread_queue_node_t) * queue->nodes_size);
             
             if(queue->nodes[queue->nodes_pos] == NULL) {
                 if(status)
@@ -831,18 +826,12 @@ mythread_queue_node_t * mythread_queue_node_malloc_limit(mythread_t *mythread, m
     
     queue->nodes_uses++;
     
-    mythread_queue_node_t *qnode = &queue->nodes[queue->nodes_pos][queue->nodes_length];
-    
-    qnode->text  = text;
-    qnode->begin = begin;
-    
-    return qnode;
+    return &queue->nodes[queue->nodes_pos][queue->nodes_length];
 }
 
 #ifndef MyHTML_BUILD_WITHOUT_THREADS
 
-mythread_queue_node_t * mythread_queue_node_malloc_round(mythread_t *mythread, mythread_queue_list_entry_t *entry,
-                                                              const char* text, size_t begin, myhtml_status_t *status)
+mythread_queue_node_t * mythread_queue_node_malloc_round(mythread_t *mythread, mythread_queue_list_entry_t *entry, myhtml_status_t *status)
 {
     mythread_queue_t* queue = entry->queue;
     
@@ -857,12 +846,7 @@ mythread_queue_node_t * mythread_queue_node_malloc_round(mythread_t *mythread, m
     else
         queue->nodes_uses++;
     
-    mythread_queue_node_t *qnode = &queue->nodes[queue->nodes_pos][queue->nodes_length];
-    
-    qnode->text  = text;
-    qnode->begin = begin;
-    
-    return qnode;
+    return &queue->nodes[queue->nodes_pos][queue->nodes_length];
 }
 
 #endif /* MyHTML_BUILD_WITHOUT_THREADS */
